#include "board_def.h"

BleKeyboard bleKeyboard("3Plus", "M0dular", 100);

struct tm timeInfo;

Button2 button1, button2, button3, button4;

// Change these two numbers to the pins connected to your encoder.
//   Best Performance: both pins have interrupt capability
//   Good Performance: only the first pin has interrupt capability
//   Low Performance:  neither pin has interrupt capability
Encoder myEnc(PIN_EC_A, PIN_EC_B);

int rst = PIN_RST;

int mosi = PIN_MOSI;
int sclk = PIN_SCLK;
int dc = PIN_DC;

int cs1 = PIN_CS1;
int cs2 = PIN_CS2;
int cs3 = PIN_CS3;

/* More data bus class: https://github.com/moononournation/Arduino_GFX/wiki/Data-Bus-Class */
Arduino_DataBus *bus1 = new Arduino_ESP32SPI(dc /*dc*/, cs1 /*cs*/, sclk /*sclk*/, mosi /*mosi*/);
// Arduino_DataBus *bus1 = new Arduino_SWSPI(dc /*dc*/, cs1 /*cs*/, sclk /*sclk*/, mosi /*mosi*/,GFX_NOT_DEFINED /*miso*/);
/* More display class: https://github.com/moononournation/Arduino_GFX/wiki/Display-Class */
Arduino_GFX *gfx1 = new Arduino_SSD1331(bus1, -1, 0 /* rotation */);

/* More data bus class: https://github.com/moononournation/Arduino_GFX/wiki/Data-Bus-Class */
Arduino_DataBus *bus2 = new Arduino_ESP32SPI(dc /*dc*/, cs2 /*cs*/, sclk /*sclk*/, mosi /*mosi*/);
// Arduino_DataBus *bus2 = new Arduino_SWSPI(dc /*dc*/, cs2 /*cs*/, sclk /*sclk*/, mosi /*mosi*/,GFX_NOT_DEFINED /*miso*/);

/* More display class: https://github.com/moononournation/Arduino_GFX/wiki/Display-Class */
Arduino_GFX *gfx2 = new Arduino_SSD1331(bus2, -1, 0 /* rotation */);

/* More data bus class: https://github.com/moononournation/Arduino_GFX/wiki/Data-Bus-Class */
Arduino_DataBus *bus3 = new Arduino_ESP32SPI(dc /*dc*/, cs3 /*cs*/, sclk /*sclk*/, mosi /*mosi*/);
// Arduino_DataBus *bus3 = new Arduino_SWSPI(dc /*dc*/, cs3 /*cs*/, sclk /*sclk*/, mosi /*mosi*/,GFX_NOT_DEFINED /*miso*/);

/* More display class: https://github.com/moononournation/Arduino_GFX/wiki/Display-Class */
Arduino_GFX *gfx3 = new Arduino_SSD1331(bus3, -1, 0 /* rotation */);

/*******************************************************************************
 * End of Arduino_GFX setting
 ******************************************************************************/

Arduino_GFX *gfx[3] = {gfx1, gfx2, gfx3};

PNG png;

int16_t w, h, xOffset, yOffset;

// Functions to access a file on the SD card
File pngFile;

void *myOpen(const char *filename, int32_t *size)
{

    pngFile = SPIFFS.open(filename, "r");

    if (!pngFile || pngFile.isDirectory())
    {
        // Serial.println(F("ERROR: Failed to open  file for reading"));
        // gfx2->println(F("ERROR: Failed to open  file for reading"));
    }
    else
    {
        *size = pngFile.size();
        // Serial.printf("Opened '%s', size: %d\n", filename, *size);
    }

    return &pngFile;
}

void myClose(void *handle)
{
    if (pngFile)
        pngFile.close();
}

int32_t myRead(PNGFILE *handle, uint8_t *buffer, int32_t length)
{
    if (!pngFile)
        return 0;
    return pngFile.read(buffer, length);
}

int32_t mySeek(PNGFILE *handle, int32_t position)
{
    if (!pngFile)
        return 0;
    return pngFile.seek(position);
}

// Function to draw pixels to the display
void PNGDraw1(PNGDRAW *pDraw)
{
    uint16_t usPixels[320];
    uint8_t usMask[320];

    // Serial.printf("Draw pos = 0,%d. size = %d x 1\n", pDraw->y, pDraw->iWidth);
    png.getLineAsRGB565(pDraw, usPixels, PNG_RGB565_LITTLE_ENDIAN, 0x00000000);
    png.getAlphaMask(pDraw, usMask, 1);
    gfx1->draw16bitRGBBitmap(xOffset, yOffset + pDraw->y, usPixels, usMask, pDraw->iWidth, 1);
}

void PNGDraw2(PNGDRAW *pDraw)
{
    uint16_t usPixels[320];
    uint8_t usMask[320];

    // Serial.printf("Draw pos = 0,%d. size = %d x 1\n", pDraw->y, pDraw->iWidth);
    png.getLineAsRGB565(pDraw, usPixels, PNG_RGB565_LITTLE_ENDIAN, 0x00000000);
    png.getAlphaMask(pDraw, usMask, 1);
    gfx2->draw16bitRGBBitmap(xOffset, yOffset + pDraw->y, usPixels, usMask, pDraw->iWidth, 1);
}

void PNGDraw3(PNGDRAW *pDraw)
{
    uint16_t usPixels[320];
    uint8_t usMask[320];

    // Serial.printf("Draw pos = 0,%d. size = %d x 1\n", pDraw->y, pDraw->iWidth);
    png.getLineAsRGB565(pDraw, usPixels, PNG_RGB565_LITTLE_ENDIAN, 0x00000000);
    png.getAlphaMask(pDraw, usMask, 1);
    gfx3->draw16bitRGBBitmap(xOffset, yOffset + pDraw->y, usPixels, usMask, pDraw->iWidth, 1);
}

void myDrawPNG(int16_t x, int16_t y, const char *path, uint8_t oled_index)
{
    unsigned long start = millis();
    int rc;
    PNG_DRAW_CALLBACK *pfnDraw = NULL;
    switch (oled_index)
    {
    case 0:
        pfnDraw = PNGDraw1;
        break;
    case 1:
        pfnDraw = PNGDraw2;
        break;
    case 2:
        pfnDraw = PNGDraw3;
        break;
    default:
        break;
    }

    rc = png.open(path, myOpen, myClose, myRead, mySeek, pfnDraw);
    if (rc == PNG_SUCCESS)
    {
        // int16_t pw = png.getWidth();
        // int16_t ph = png.getHeight();

        xOffset = x; //+(OLED_WIDTH - pw) / 2;
        yOffset = y; //+(OLED_HEIGHT - ph) / 2;

        rc = png.decode(NULL, 0);

        // Serial.printf("Draw offset: (%d, %d), time used: %lu\r\n", xOffset, yOffset, millis() - start);
        // Serial.printf("image specs: (%d x %d), %d bpp, pixel type: %d\r\n", png.getWidth(), png.getHeight(), png.getBpp(), png.getPixelType());
        png.close();
        // delay(5000);
    }
    else
    {
        gfx[oled_index]->setFont(u8g_font_5x7);
        gfx[oled_index]->setCursor(x, y + 20);
        gfx[oled_index]->println("png failed!");
    }
}

void board_init()
{
    Serial.begin(115200);
    delay(10);
    Serial.println("start");

    if (!SPIFFS.begin())
    {
        Serial.println(F("ERROR: File System Mount Failed!"));
    }
    else
    {
    }

    pinMode(rst, OUTPUT);
    digitalWrite(rst, 1);
    delay(100);
    digitalWrite(rst, 0);
    delay(100);
    digitalWrite(rst, 1);
    delay(100);

    gfx1->begin();
    gfx1->fillScreen(BLACK);
    w = gfx1->width(), h = gfx1->height();

    gfx2->begin();
    gfx2->fillScreen(BLACK);

    // bus2->sendCommand(SSD1331_CONTRASTA); // 0x81
    // bus2->sendCommand(0xff);
    // bus2->sendCommand(SSD1331_CONTRASTB); // 0x82
    // bus2->sendCommand(0xff);
    // bus2->sendCommand(SSD1331_CONTRASTC); // 0x83
    // bus2->sendCommand(0xff);

    gfx3->begin();
    gfx3->fillScreen(BLACK);

    gfx1->setTextColor(GREEN);
    gfx2->setTextColor(GREEN);
    gfx3->setTextColor(GREEN);
    gfx3->setFont(u8g2_font_5x7_mr);

    myDrawPNG(0, 0, "/name.png", 0); // delay(5000);
    myDrawPNG(0, 0, "/ver.png", 1);
    myDrawPNG(0, 0, "/THREE.png", 2); // delay(5000);

    delay(1000);
    for (uint8_t i = 0; i < 32; i++)
    {
        // myDrawPNG(0, 0-i, "/THREE.png", 0);
        gfx3->drawRect(0, 32 - i, OLED_WIDTH, i * 2, BLACK);

        delay(20);
    }

    gfx3->setCursor(0, 7);
    gfx3->printf("HW: %s\r\n", VER_HW);
    gfx3->printf("SW: %s\r\n", VER_SW);

    gfx3->printf("Free rom: %dKB\r\n", (SPIFFS.totalBytes() - SPIFFS.usedBytes()) / 1024);

    WiFi.mode(WIFI_STA);
    WiFi.begin(ssid, password);

    while (WiFi.status() != WL_CONNECTED)
    {
        gfx3->print('.');
        delay(200);
    }
    gfx3->println(" \r\nConnected ");
    gfx3->println("IP address: ");
    gfx3->setTextColor(GREENYELLOW);
    gfx3->println(WiFi.localIP()); // 显示连接WIFI后的IP地址
    gfx3->setTextColor(GREEN);
    gfx3->print("RSSI: ");      // 显示连接WIFI后的IP地址
    gfx3->println(WiFi.RSSI()); // 显示连接WIFI后的IP地址
    delay(1500);
    gfx3->print("time updating"); // 显示连接WIFI后的IP地址
    configTime(8 * 3600, 0, NTP1, NTP2, NTP3);
    while (!getLocalTime(&timeInfo))
    {
        if (gfx3->getCursorY() > (63 - 7))
        {
            gfx3->setCursor(0, 7);
            gfx3->fillScreen(BLACK);
        }
        gfx3->print('.');
        delay(200);
    }

    // WiFi.mode(WIFI_OFF);

    bleKeyboard.begin();
}
